<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

/**
 * GSF Tumblr Counter.
 *
 * @package /G5P_Tumblr_Counter
 * @category Counter
 */
class G5P_Follower_Tumblr_Counter extends G5P_Followers_Counter
{

    /**
     * Counter ID.
     *
     * @var string
     */
    public $id = 'tumblr';

    /**
     * API URL.
     *
     * @var string
     */
    protected $api_url = 'https://api.tumblr.com/v2/blog/%s/followers';

    /**
     * Authorization.
     *
     * @param  string $hostname hostname.
     * @param  string $consumer_key Ccustomer key.
     * @param  string $consumer_secret Customer secret.
     * @param  string $oauth_access_token OAuth access token.
     * @param  string $oauth_access_token_secret OAuth access token secret.
     *
     * @return string
     */
    protected function authorization($hostname, $consumer_key, $consumer_secret, $oauth_access_token, $oauth_access_token_secret)
    {
        $signature = $this->signature($hostname, $consumer_key, $consumer_secret, $oauth_access_token, $oauth_access_token_secret);

        return $this->header($signature);
    }

    /**
     * Build the Signature base string.
     *
     * @param  string $url Request URL.
     * @param  string $query Request query.
     * @param  string $method Request method.
     * @param  string $params OAuth params.
     *
     * @return string          OAuth Signature base.
     */
    private function signature_base_string($url, $method, $params)
    {
        $return = array();
        ksort($params);

        foreach ($params as $key => $value) {
            $return[] = $key . '=' . $value;
        }

        return $method . '&' . rawurlencode($url) . '&' . rawurlencode(implode('&', $return));
    }

    /**
     * Build the OAuth Signature.
     *
     * @param  string $hostname hostname.
     * @param  string $consumer_key Twitter customer key.
     * @param  string $consumer_secret Twitter customer secret.
     * @param  string $oauth_access_token OAuth access token.
     * @param  string $oauth_access_token_secret OAuth access token secret.
     *
     * @return array                             OAuth signature params.
     */
    private function signature($hostname, $consumer_key, $consumer_secret, $oauth_access_token, $oauth_access_token_secret)
    {
        $oauth = array(
            'oauth_consumer_key' => $consumer_key,
            'oauth_nonce' => hash_hmac('sha1', time(), '1', false),
            'oauth_signature_method' => 'HMAC-SHA1',
            'oauth_token' => $oauth_access_token,
            'oauth_timestamp' => time(),
            'oauth_version' => '1.0'
        );

        $base_info = $this->signature_base_string(sprintf($this->api_url, $hostname), 'GET', $oauth);
        $composite_key = rawurlencode($consumer_secret) . '&' . rawurlencode($oauth_access_token_secret);
        $oauth_signature = base64_encode(hash_hmac('sha1', $base_info, $composite_key, true));
        $oauth['oauth_signature'] = $oauth_signature;

        return $oauth;
    }

    /**
     * Build the header.
     *
     * @param  array $signature OAuth signature.
     *
     * @return string           OAuth Authorization.
     */
    public function header($signature)
    {
        $return = 'OAuth ';
        $values = array();

        foreach ($signature as $key => $value) {
            $values[] = $key . '="' . rawurlencode($value) . '"';
        }

        $return .= implode(', ', $values);

        return $return;
    }

    /**
     * Test the counter is available.
     *
     * @param  array $data
     *
     * @return bool
     */
    public function is_available($data)
    {
        return !empty($data['tumblr_hostname']) && !empty($data['tumblr_consumer_key']) && !empty($data['tumblr_consumer_secret']) && !empty($data['tumblr_token']) && !empty($data['tumblr_token_secret']);
    }

    /**
     * Get the total.
     *
     * @param  array $data
     *
     * @return int
     */
    public function get_total($data)
    {
        if (is_array($data)&&count($data) > 0) {
            if ($this->is_available($data)) {
                $hostname = str_replace(array('http:', 'https:', '/'), '', sanitize_text_field($data['tumblr_hostname']));

                $params = array(
                    'method' => 'GET',
                    'timeout' => 60,
                    'headers' => array(
                        'Content-Type' => 'application/x-www-form-urlencoded',
                        'Authorization' => $this->authorization(
                            $hostname,
                            $data['tumblr_consumer_key'],
                            $data['tumblr_consumer_secret'],
                            $data['tumblr_token'],
                            $data['tumblr_token_secret']
                        )
                    )
                );

                $this->connection = wp_remote_get(sprintf($this->api_url, $hostname), $params);

                if (is_wp_error($this->connection) || (isset($this->connection['response']['code']) && 200 != $this->connection['response']['code'])) {
                    $this->total = 0;
                } else {
                    $_data = json_decode($this->connection['body'], true);

                    if (isset($_data['response']['total_users'])) {
                        $count = intval($_data['response']['total_users']);

                        $this->total = $count;
                    } else {
                        $this->total = 0;
                    }
                }
            }
        }

        return $this->total;
    }

    /**
     * Get conter view.
     *
     * @param  array $instance Widget settings.
     *
     * @return string
     */
    public function get_view($instance)
    {
        $tumblr_hostname = !empty($instance['tumblr_hostname']) ? $instance['tumblr_hostname'] : '';
        $tumblr_consumer_key = !empty($instance['tumblr_consumer_key']) ? $instance['tumblr_consumer_key'] : '';
        $tumblr_consumer_secret = !empty($instance['tumblr_consumer_secret']) ? $instance['tumblr_consumer_secret'] : '';
        $tumblr_token = !empty($instance['tumblr_token']) ? $instance['tumblr_token'] : '';
        $tumblr_token_secret = !empty($instance['tumblr_token_secret']) ? $instance['tumblr_token_secret'] : '';
        $data = array(
            'tumblr_hostname' => $tumblr_hostname,
            'tumblr_consumer_key' => $tumblr_consumer_key,
            'tumblr_consumer_secret' => $tumblr_consumer_secret,
            'tumblr_token' => $tumblr_token,
            'tumblr_token_secret' => $tumblr_token_secret
        );
        $icon = !empty($instance['tumblr_icon']) ? $instance['tumblr_icon'] : '';
        $icon_color = !empty($instance['tumblr_icon_color']) ? $instance['tumblr_icon_color'] : '';
        ob_start();
        G5P()->helper()->getTemplate('widgets/followers/inc/templates/get-view-li', array(
                'id' => $this->id,
                'data' => $data,
                'icon' => $icon,
                'url' => $tumblr_hostname,
                'label' => __('Followers', 'achilles-framework'),
                'instance' => $instance,
                'icon_color' => $icon_color,
            )
        );
        $content = ob_get_clean();
        return $content;
    }
}
