<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

/**
 * GSF Twitter Counter.
 *
 * @package /G5P_Twitter_Counter
 * @category Counter
 */
class G5P_Follower_Twitter_Counter extends G5P_Followers_Counter
{

    /**
     * Counter ID.
     *
     * @var string
     */
    public $id = 'twitter';

    /**
     * API URL.
     *
     * @var string
     */
    protected $api_url = 'https://api.twitter.com/1.1/users/show.json';

    /**
     * Authorization.
     *
     * @param  string $user Twitter username.
     * @param  string $consumer_key Twitter customer key.
     * @param  string $consumer_secret Twitter customer secret.
     * @param  string $oauth_access_token OAuth access token.
     * @param  string $oauth_access_token_secret OAuth access token secret.
     *
     * @return string
     */
    protected function authorization($user, $consumer_key, $consumer_secret, $oauth_access_token, $oauth_access_token_secret)
    {
        $query = 'screen_name=' . $user;
        $signature = $this->signature($query, $consumer_key, $consumer_secret, $oauth_access_token, $oauth_access_token_secret);

        return $this->header($signature);
    }

    /**
     * Build the Signature base string.
     *
     * @param  string $url Request URL.
     * @param  string $query Request query.
     * @param  string $method Request method.
     * @param  string $params OAuth params.
     *
     * @return string          OAuth Signature base.
     */
    private function signature_base_string($url, $query, $method, $params)
    {
        $return = array();
        ksort($params);

        foreach ($params as $key => $value) {
            $return[] = $key . '=' . $value;
        }

        return $method . '&' . rawurlencode($url) . '&' . rawurlencode(implode('&', $return)) . '%26' . rawurlencode($query);
    }

    /**
     * Build the OAuth Signature.
     *
     * @param  string $query Request query.
     * @param  string $consumer_key Twitter customer key.
     * @param  string $consumer_secret Twitter customer secret.
     * @param  string $oauth_access_token OAuth access token.
     * @param  string $oauth_access_token_secret OAuth access token secret.
     *
     * @return array                             OAuth signature params.
     */
    private function signature($query, $consumer_key, $consumer_secret, $oauth_access_token, $oauth_access_token_secret)
    {
        $oauth = array(
            'oauth_consumer_key' => $consumer_key,
            'oauth_nonce' => hash_hmac('sha1', time(), '1', false),
            'oauth_signature_method' => 'HMAC-SHA1',
            'oauth_token' => $oauth_access_token,
            'oauth_timestamp' => time(),
            'oauth_version' => '1.0'
        );

        $base_info = $this->signature_base_string($this->api_url, $query, 'GET', $oauth);
        $composite_key = rawurlencode($consumer_secret) . '&' . rawurlencode($oauth_access_token_secret);
        $oauth_signature = base64_encode(hash_hmac('sha1', $base_info, $composite_key, true));
        $oauth['oauth_signature'] = $oauth_signature;

        return $oauth;
    }

    /**
     * Build the header.
     *
     * @param  array $signature OAuth signature.
     *
     * @return string           OAuth Authorization.
     */
    public function header($signature)
    {
        $return = 'OAuth ';
        $values = array();

        foreach ($signature as $key => $value) {
            $values[] = $key . '="' . rawurlencode($value) . '"';
        }

        $return .= implode(', ', $values);

        return $return;
    }

    /**
     * Test the counter is available.
     *
     * @param  array $data
     *
     * @return bool
     */
    public function is_available($data)
    {
        return !empty($data['twitter_user']) && !empty($data['twitter_consumer_key']) && !empty($data['twitter_consumer_secret']) && !empty($data['twitter_access_token']) && !empty($data['twitter_access_token_secret']);
    }

    /**
     * Get the total.
     *
     * @param  array $data
     *
     * @return int
     */
    public function get_total($data)
    {
        if (is_array($data)&&count($data) > 0) {
            if ($this->is_available($data)) {
                $user = $data['twitter_user'];
                $params = array(
                    'method' => 'GET',
                    'timeout' => 60,
                    'headers' => array(
                        'Content-Type' => 'application/x-www-form-urlencoded',
                        'Authorization' => $this->authorization(
                            $user,
                            $data['twitter_consumer_key'],
                            $data['twitter_consumer_secret'],
                            $data['twitter_access_token'],
                            $data['twitter_access_token_secret']
                        )
                    )
                );

                $this->connection = wp_remote_get($this->api_url . '?screen_name=' . $user, $params);

                if (is_wp_error($this->connection)) {
                    $this->total = 0;
                } else {
                    $_data = json_decode($this->connection['body'], true);

                    if (isset($_data['followers_count'])) {
                        $count = intval($_data['followers_count']);

                        $this->total = $count;
                    } else {
                        $this->total = 0;
                    }
                }
            }
        }

        return $this->total;
    }

    /**
     * Get counter view.
     *
     * @param  array $instance Widget settings.
     *
     * @return string
     */
    public function get_view($instance)
    {
        $twitter_user = !empty($instance['twitter_username']) ? $instance['twitter_username'] : '';
        $twitter_consumer_key = !empty($instance['twitter_consumer_key']) ? $instance['twitter_consumer_key'] : '';
        $twitter_consumer_secret = !empty($instance['twitter_consumer_secret']) ? $instance['twitter_consumer_secret'] : '';
        $twitter_access_token = !empty($instance['twitter_access_token']) ? $instance['twitter_access_token'] : '';
        $twitter_access_token_secret = !empty($instance['twitter_access_token_secret']) ? $instance['twitter_access_token_secret'] : '';
        $data = array(
            'twitter_user' => $twitter_user,
            'twitter_consumer_key' => $twitter_consumer_key,
            'twitter_consumer_secret' => $twitter_consumer_secret,
            'twitter_access_token' => $twitter_access_token,
            'twitter_access_token_secret' => $twitter_access_token_secret
        );
        $twitter_user = !empty($instance['twitter_username']) ? $instance['twitter_username'] : '';
        $icon = !empty($instance['twitter_icon']) ? $instance['twitter_icon'] : '';
        $icon_color = !empty($instance['twitter_icon_color']) ? $instance['twitter_icon_color'] : '';
        ob_start();
        G5P()->helper()->getTemplate('widgets/followers/inc/templates/get-view-li', array(
                'id' => $this->id,
                'data' => $data,
                'icon' => $icon,
                'url' => 'https://twitter.com/' . $twitter_user,
                'label' => __('Followers', 'achilles-framework'),
                'instance' => $instance,
                'icon_color' => $icon_color,
            )
        );
        $content = ob_get_clean();
        return $content;
    }
}
